﻿using CloudReporting.DataRepository;
using CloudReporting.Reports;
using Microsoft.Reporting.WebForms;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Queue;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Reflection;
using System.Web.Http;

namespace CloudReporting.MVC.Controllers
{
    public class ReportController : ApiController
    {
        private static string _servicesBusConnectionString = ConfigurationManager.ConnectionStrings["AzureWebJobsServiceBus"].ConnectionString;

        private static NamespaceManager _namespaceMgr = null;
        protected static NamespaceManager NamespaceMgr
        {
            get
            {
                if (_namespaceMgr == null) _namespaceMgr = NamespaceManager.CreateFromConnectionString(_servicesBusConnectionString);
                return _namespaceMgr;
            }
        }

        private static MessagingFactory _msgFactory = null;
        public static MessagingFactory MsgFactory
        {
            get
            {
                if (_msgFactory == null) _msgFactory = MessagingFactory.Create(NamespaceMgr.Address, NamespaceMgr.Settings.TokenProvider);
                return _msgFactory;
            }
        }

        public GetOrdersResponse Get(string category)
        {
            try
            {
                var result = new GetOrdersResponse();
                var repository = new Repository();
                result.Orders = repository.GetFakeOrdersForProductCategoryReport(category);
                result.Success = true;
                return result;
            }
            catch
            {
                //Log the error here.
                //Return a user-friendly message to the caller.
                return new GetOrdersResponse
                {
                    Success = false,
                    FailureInformation = "Could not get orders."
                };
            }
        }

        public GetOrdersAsFileResponse Get(string category, string format)
        {
            try
            {
                var response = new GetOrdersAsFileResponse();
                var data = Get(category);
                if (!data.Success)
                {
                    response.Success = false;
                    response.FailureInformation = data.FailureInformation;
                    return response;
                }

                var rv = new ReportViewer();
                rv.ProcessingMode = ProcessingMode.Local;
                rv.Reset();
                var assembly = Assembly.Load("CloudReporting.Reports");
                var stream = assembly.GetManifestResourceStream(
                "CloudReporting.Reports.OrdersForProductCategory.rdlc");
                rv.LocalReport.LoadReportDefinition(stream);
                rv.LocalReport.DataSources.Clear();
                rv.LocalReport.DataSources.Add(new ReportDataSource("DataSet1", data.Orders));
                byte[] streambytes = null;
                var mimeType = string.Empty;
                var encoding = string.Empty;
                var fileExtension = string.Empty;
                string[] streamIds = null;
                Warning[] warnings = null;
                streambytes = rv.LocalReport.Render(format, null, out mimeType,
                out encoding, out fileExtension, out streamIds, out warnings);
                rv.LocalReport.ReleaseSandboxAppDomain();  //Report viewer
                                                           //uses a lot more memory and will give out of memory exceptions
                                                           //if you don't call this.
                response.FileContents = streambytes;
                response.MimeType = mimeType;
                response.Encoding = encoding;
                response.FileName = "OrdersForProductCategory." + fileExtension;

                response.Success = data.Success;
                return response;
            }
            catch (Exception ex)
            {
                //Log the error here.
                //Return a user-friendly message to the caller.
                return new GetOrdersAsFileResponse
                {
                    Success = false,
                    FailureInformation = "Could not get orders report."
                };
            }
        }

        public EmailReportResponse Get(string category, string format, string email)
        {
            try
            {
                var response = new EmailReportResponse();
                var parameters = new ReportParameters
                {
                    Category = category,
                    Format = format,
                    To = email,
                    From = "system@cloudreporting.com",
                    Subject = "Here is your report",
                };

                var storageAccount = CloudStorageAccount.Parse(ConfigurationManager.ConnectionStrings["AzureWebJobsStorage"].ConnectionString);
                var queueClient = storageAccount.CreateCloudQueueClient();
                var queue = queueClient.GetQueueReference("report-request");
                queue.AddMessage(new CloudQueueMessage(JsonConvert.SerializeObject(parameters)));

                response.Success = true;
                return response;
            }
            catch (Exception ex)
            {
                //Log the error here.
                return new EmailReportResponse
                {
                    Success = false,
                    FailureInformation = "Could not queue report.",
                };
            }
        }

        public EmailReportResponse Get(string category, string format, string email, Guid sessionId)
        {
            try
            {
                var response = new EmailReportResponse();
                var parameters = new ReportParameters
                {
                    Category = category,
                    Format = format,
                    To = email,
                    From = "system@cloudreporting.com",
                    Subject = "Here is your report",
                    SessionId = sessionId,
                };

                var queueClient = MsgFactory.CreateQueueClient("report-request");
                queueClient.Send(new BrokeredMessage(parameters));

                response.Success = true;
                return response;
            }
            catch (Exception ex)
            {
                //Log the error here.
                return new EmailReportResponse
                {
                    Success = false,
                    FailureInformation = "Could not queue report.",
                };
            }
        }

        public GetQueuedReportStatusResponse GetQueuedReportStatus(Guid sessionId)
        {
            try
            {
                var response = new GetQueuedReportStatusResponse();

                var queueClient = MsgFactory.CreateQueueClient("report-response", ReceiveMode.ReceiveAndDelete);
                var messageSession = queueClient.AcceptMessageSession(sessionId.ToString(), new TimeSpan(100));
                var brokeredMessage = messageSession.Receive(new TimeSpan(1));
                while (brokeredMessage != null)
                {
                    response.Messages.Add(brokeredMessage.GetBody<ReportStatusResponse>());
                    brokeredMessage = messageSession.Receive(new TimeSpan(1));
                }
                messageSession.Close();

                response.Success = true;
                return response;
            }
            catch (Exception ex)
            {
                //Log the error here.
                return new GetQueuedReportStatusResponse
                {
                    Success = false,
                    FailureInformation = "Could not get queued report status."
                };
            }
        }

    }

    public class GetQueuedReportStatusResponse
    {
        public bool Success { get; set; } = false;
        public string FailureInformation { get; set; } = string.Empty;
        public List<ReportStatusResponse> Messages { get; set; } = new List<ReportStatusResponse>();
    }

    public class EmailReportResponse
    {
        public bool Success { get; set; } = false;
        public string FailureInformation { get; set; } = string.Empty;
    }

    public class GetOrdersResponse
    {
        public bool Success { get; set; } = false;
        public string FailureInformation { get; set; } = string.Empty;
        public List<OrderInformation> Orders { get; set; } = new List<OrderInformation>();
    }

    public class GetOrdersAsFileResponse
    {
        public bool Success { get; set; } = false;
        public string FailureInformation { get; set; } = string.Empty;
        public byte[] FileContents { get; set; } = null;
        public string MimeType { get; set; }
        public string Encoding { get; set; }
        public string FileName { get; set; }
    }

}