﻿using CloudReporting.DataRepository;
using CloudReporting.Reports;
using Microsoft.Azure.WebJobs;
using Microsoft.Reporting.WebForms;
using Microsoft.ServiceBus.Messaging;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Mail;
using System.Reflection;
using System.Threading;

namespace CloudReporting.WebJobs
{
    public static class RunReport
    {
        public static void ReadReportRequestQueue([ServiceBusTrigger("report-request")] ReportParameters request,
            [ServiceBus("report-response")] ICollector<BrokeredMessage> responseQueue, TextWriter log)
        {
            var sessionId = request.SessionId.ToString();
            var msg = "Read from Report Request Queue.";
            log.WriteLine(msg);
            var queueMsg = new ReportStatusResponse { Message = msg };
            responseQueue.Add(new BrokeredMessage(queueMsg) { SessionId = sessionId });

            var pdf = getReport(request.Category, request.Format, responseQueue, sessionId, log);
            if (pdf == null)
            {
                msg = "Failed to send report to: " + request.To;
                log.WriteLine(msg);
                queueMsg = new ReportStatusResponse { Message = msg, IsProcessComplete = true };
                responseQueue.Add(new BrokeredMessage(queueMsg) { SessionId = sessionId });

                sendEmail(request.From, "helpdesk@mycompany.com", "Emailing failed", "Please check the WebJob logs.", null);
                return;
            }

            sendEmail(request.From, request.To, request.Subject, "You must have a PDF reader installed on your machine to open the report.", pdf);

            msg = "Successfully sent report to: " + request.To;
            log.WriteLine(msg);
            queueMsg = new ReportStatusResponse { Message = msg, IsProcessComplete = true };
            responseQueue.Add(new BrokeredMessage(queueMsg) { SessionId = sessionId });
        }


        private static GetOrdersResponse getData(string category, ICollector<BrokeredMessage> responseQueue, string sessionId)
        {
            var result = new GetOrdersResponse();
            try
            {
                var repository = new Repository();
                result.Orders = repository.GetFakeOrdersForProductCategoryReport(category);
                result.Success = true;

                var msg = "Gathering data for report.";
                for (int i = 1; i < 11; i++)
                {
                    var queueMsg = new ReportStatusResponse { Message = msg, Count = i, TotalCount = 10 };
                    responseQueue.Add(new BrokeredMessage(queueMsg) { SessionId = sessionId });
                    Thread.Sleep(2000);
                }
            }
            catch
            {
                //Log the error here.
                result.Success = false;
            }
            return result;
        }

        private static byte[] getReport(string category, string format, ICollector<BrokeredMessage> responseQueue, string sessionId, TextWriter log)
        {
            try
            {
                var data = getData(category, responseQueue, sessionId);
                if (!data.Success)
                {
                    log.WriteLine("Could not retrieve data for the report.");
                    return null;
                }

                var queueMsg = new ReportStatusResponse { Message = "Generating report." };
                responseQueue.Add(new BrokeredMessage(queueMsg) { SessionId = sessionId });

                var rv = new ReportViewer();
                rv.ProcessingMode = ProcessingMode.Local;
                rv.Reset();
                var assembly = Assembly.Load("CloudReporting.Reports");
                var stream = assembly.GetManifestResourceStream(
                "CloudReporting.Reports.OrdersForProductCategory.rdlc");
                rv.LocalReport.LoadReportDefinition(stream);
                rv.LocalReport.DataSources.Clear();
                rv.LocalReport.DataSources.Add(new ReportDataSource("DataSet1", data.Orders));
                byte[] streambytes = null;
                var mimeType = string.Empty;
                var encoding = string.Empty;
                var fileExtension = string.Empty;
                string[] streamIds = null;
                Warning[] warnings = null;
                streambytes = rv.LocalReport.Render(format, null, out mimeType,
                out encoding, out fileExtension, out streamIds, out warnings);
                rv.LocalReport.ReleaseSandboxAppDomain();  //Report viewer
                                                           //uses a lot more memory and will give out of memory exceptions
                                                           //if you don't call this.
                return streambytes;
            }
            catch (Exception ex)
            {
                log.WriteLine("An exception occurred rendering the report.");
                log.WriteLine(ex.Message);
                return null;
            }

        }

        private static void sendEmail(string from, string to, string subject, string body, byte[] pdf)
        {
            var mail = new MailMessage(from, to);
            var client = new SmtpClient();
            client.Port = 25;
            client.DeliveryMethod = SmtpDeliveryMethod.Network;
            client.UseDefaultCredentials = false;
            client.Host = "smtp.google.com";    //TODO: This is not a real mail server.
            //client.Credentials = new NetworkCredential("mylogin", "mypassword");
            mail.Subject = subject;
            mail.Body = body;
            if (pdf != null)
            {
                var stream = new MemoryStream(pdf);
                var attachment = new Attachment(stream, "OrdersForProductCategory.PDF");
                mail.Attachments.Add(attachment);
            }
            client.Send(mail);
        }
    }

    public class GetOrdersResponse
    {
        public bool Success { get; set; } = false;
        public string FailureInformation { get; set; } = string.Empty;
        public List<OrderInformation> Orders { get; set; } = new List<OrderInformation>();
    }
}